@@ -2,11 +2,11 @@ __author__ = "Swas.py"
 __title__ = "vscode"
 __license__ = "MIT"
 __copyright__ = "Copyright 2021 Swas.py"
-__version__ = "1.4.8"
+__version__ = "1.4.9"
 
 from . import window
 from .compiler import build
 from .extension import Extension
 from .envMethods import env
-from . import _types as ext
-from ._types import *
+from . import types as ext
+from .types import *
